unit uTestCase;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeEngine, CurvFitt, Series, ExtCtrls, TeeProcs, Chart, TeeEdit;

type
  TLifeTemp = class(TForm)
    chLifeTemp: TChart;
    LabData: TPointSeries;
    CurveFit: TFastLineSeries;
    TTeeFunction1: TCurveFittingFunction;
    ChartEditor1: TChartEditor;
    SmoothFit: TFastLineSeries;
    serUpper: TPointSeries;
    serlower: TPointSeries;
    procedure FormCreate(Sender: TObject);
    procedure chLifeTempDblClick(Sender: TObject);
    procedure chLifeTempAfterDraw(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  LifeTemp: TLifeTemp;

implementation

uses dmChartUtils, math;

{$R *.dfm}


procedure TLifeTemp.FormCreate(Sender: TObject);

var i : integer; xMax, yMax : double;

begin
  tTeeFunction1.PolyDegree := 2;
  with LabData do begin
(**)  //  Orig: 3.88k, 74.3kj
    AddXY( 1.808, log10( 85 ));
    AddXY( 1.876, log10( 130 ));
    AddXY( 1.949, log10( 280 ));
    AddXY( 2.070, log10( 840 ));
(**)
(*   // Maggie 3.4k, 65.2kj; identical agreement
    AddXY( 1.808, log10( 110.74 ));
    AddXY( 1.876, log10( 174.03 ));
    AddXY( 1.949, log10( 362.21 ));
    AddXY( 2.070, log10( 830.00 ));
*)
(*
    AddXY( 210, 830 );
    AddXY( 240, 362.2143 );
    AddXY( 260, 174.0279 );
    AddXY( 280, 110.742 );
*)
(*
    AddXY( 1000/2.1 - 273, power( 10, 2.1 ));
    AddXY( 1000/2.0 - 273, power( 10, 2.0 ));
    AddXY( 1000/1.9 - 273, power( 10, 1.9 ));
    AddXY( 1000/1.8 - 273, power( 10, 1.8 ));
*)
    for i := 0 to count-1 do begin
      serUpper.AddXY( LabData.xValue[ i ], 1.02*LabData.yValue[ i ]);
      serLower.AddXY( LabData.xValue[ i ], 0.98*LabData.yValue[ i ]);
    end;
    chLifeTemp.LeftAxis.TitleSize := 0;
    chLifeTemp.LeftAxis.LabelsSize := 10 + canvas.TextWidth( '2000' );
    chLifeTemp.BottomAxis.TitleSize := 0;
    chLifeTemp.BottomAxis.LabelsSize := 10 + canvas.TextHeight( '2' );
  end;
  with tTeeFunction1 do begin
    AddPoints( LabData );
    yMax := log10( ChartCommon.MaxLife );
    xMax := CurveFit.MaxXValue + ( Ymax - CurveFit.MaxYvalue ) / AnswerVector[ 2 ];
    ChartCommon.TemperatureIndex := 1000/xMax - 273;
    CurveFit.AddXY( xMax, Ymax );
  end;
  chLifeTemp.LeftAxis.SetMinMax( chLifeTemp.LeftAxis.Minimum,
    log10( 110000 ));
  xMax := xMax + log10( 1.2 ) / tTeeFunction1.AnswerVector[ 2 ];
  with chLifeTemp.BottomAxis do
    SetMinMax( Minimum, xMax );
  { end }
end;  { FormCreate }

procedure TLifeTemp.chLifeTempDblClick(Sender: TObject);
begin
  ChartEditor1.Execute
end;

procedure TLifeTemp.chLifeTempAfterDraw(Sender: TObject);

var
  dy, y, yMax, yMin, slope, Ea, yy : double;
  x, dx, x1, y1, y2, dy1, t, TI : integer;
  tps : string;
  tpi : longint;

begin
  if not chLifeTemp.BottomAxis.inverted then
    exit;
  with LabData, ChartCommon do begin
    yMin := chLifeTemp.LeftAxis.minimum;
    yMax := chLifeTemp.LeftAxis.Maximum;
    yMin := GetRoundLog( MinYValue, false );
    // Possible mismatch between data ymin and axis ymin. work on this
    with chLifeTemp, canvas do begin
      y := yMin;
      tpi := round( Power( 10, y ));
      canvas.Font.Size := 8;
      Pen.Style := psDot;
      y1 := chLifeTemp.LeftAxis.IStartPos;
      y2 := chLifeTemp.LeftAxis.IEndPos;
      with LeftAxis do
      repeat
        y1 := CalcPosValue( y );
        if tpi < 10000 then
          tps := IntToStr( tpi )
        else begin
          yy := tpi;
          tps := FloatToStrf( yy, ffGeneral, 3, 4 );
        end;
        if (y1 > CalcPosValue(Maximum)) and (y1 < CalcPosValue(Minimum)) then begin
          DrawAxisLabel( PosAxis - 4, y1, 0, IntToStr( tpi ));
          MoveTo( PosAxis, y1 );
          LineTo( BottomAxis.IEndPos, y1 );
        end;
        dy := GetNextTic( y );
        y := y + dy;
        tpi := round( Power( 10, y ));
      until y >= yMax;
      with BottomAxis do begin
        x := 100;
        y1 := LeftAxis.iEndPos + 5;
        t := 300;
        repeat
          tps := IntToStr( t );
          x := CalcPosValue( 1000/(t+273));
          MoveTo( x, LeftAxis.IEndPos );
          LineTo( x, LeftAxis.IStartPos );
          dx := TextWidth( tps ) div 2;
          DrawAxisLabel( x, y1, 0, tps );
          dec( t, 20 );
        until t < 140;
      end;
      x1 := Round( BottomAxis.iEndPos * 1.02 );
      y1 := Round( LeftAxis.iEndPos * 0.3 );
      // canvas.Font.Name := 'verdana';
      canvas.Font.Size := 10;
      dy1 := TextHeight( 'S' ) +3;
      Slope := tTeeFunction1.AnswerVector[ 2 ];
      TextOut( x1, y1, 'Slope: ' +
        FloatToStrf( slope, ffGeneral, 3, 4 ) +  ' k' );
      Ea := slope * 8.31 * 2.302;  { kj/mol }
      TextOut( x1, y1 + dy1, 'Ea: ' +
        FloatToStrf( Ea, ffGeneral, 3, 4 ) + ' kj/mol' );
      TextOut( x1, y1 + 2*dy1, 'TI: ' +
        FloatToStrf( ChartCommon.TemperatureIndex, ffGeneral, 3, 4 ) +
        ' C at ' + FloatToStrf( ChartCommon.MaxLife, ffGeneral, 3, 4 ) +
        ' h.' )
      { end }
    end
  end
end;

end.
